package com.hw.jindie.controller;

import com.alibaba.fastjson2.JSONObject;
import com.hw.common.core.web.domain.AjaxResult;
import com.hw.common.log.annotation.Log;
import com.hw.common.log.enums.BusinessType;
import com.hw.jindie.service.IKingdeeErpSyncService;
import com.hw.jindie.service.impl.KingdeeErpService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Description:
 * @ProjectName:HwMes
 * @Author:xins
 * @Date:2024-01-11 17:26
 * @Version:1.0
 */
@RestController
@RequestMapping("/jindie")
public class KingdeeErpSyncController {

    @Autowired
    private IKingdeeErpSyncService kingdeeErpSyncService;

    @Autowired
    private KingdeeErpService kingdeeErpService;

    /**
     * 新增原材料入库记录
     */
//    @RequiresPermissions("wms:mobile:addrawinstock")
    @Log(title = "物料信息", businessType = BusinessType.INSERT)
    @GetMapping(("/syncMaterialInfo"))
    public AjaxResult syncMaterialInfo() {
        try {
            return AjaxResult.success(kingdeeErpSyncService.syncMaterialInfoFromErp());
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error();
        }

    }

    /**
     * 单一采购申请单保存->提交->审核
     *
     * @param params {"FMaterialId": "040200050162","FMaterialDesc": "UPVC直管","FReqQty": 20}
     * @return {"msg": "{"IsSuccess": true,"Errors": [],"SuccessEntitys": [{"Id": "154898","Number": "CGSQ010785","DIndex": 0}]}", "code": 200}
     */
    @PostMapping(("/singleSavePurchaseRequisition"))
    public AjaxResult singleSavePurchaseRequisition(@RequestBody String params) {
        try {
            return AjaxResult.success(kingdeeErpService.singleSavePurchaseRequisition(params));
        } catch (Exception e) {
            return AjaxResult.error(e.toString());
        }
    }

    /**
     * ERP通用提交接口
     *
     * @param params
     * @return
     */
    @PostMapping(("/genericCommitInterface"))
    public AjaxResult genericCommitInterface(@RequestBody JSONObject params) {
        try {
            return AjaxResult.success(kingdeeErpService.genericCommitInterface(params));
        } catch (Exception e) {
            return AjaxResult.error(e.toString());
        }
    }

    /**
     * ERP通用审核接口
     *
     * @param params
     * @return
     */
    @PostMapping(("/genericAuditInterface"))
    public AjaxResult genericAuditInterface(@RequestBody JSONObject params) {
        try {
            return AjaxResult.success(kingdeeErpService.genericAuditInterface(params));
        } catch (Exception e) {
            return AjaxResult.error(e.toString());
        }
    }

    /**
     * ERP通用查看接口
     *
     * @param params
     * @return
     */
    @PostMapping(("/genericViewInterface"))
    public String genericViewInterface(@RequestBody JSONObject params) {
        try {
            return kingdeeErpService.genericViewInterface(params);
        } catch (Exception e) {
            return e.toString();
        }
    }

    /**
     * 测试组装实体
     *
     * @param params
     * @return
     */
    @PostMapping(("/testAssemblyEntity"))
    public String testAssemblyEntity(@RequestBody String params) {
        try {
            return kingdeeErpService.testAssemblyEntity(params);
        } catch (Exception e) {
            return e.toString();
        }
    }

}
